#! /usr/bin/python

import time, sys, os, os.path, threading
import Image, ImageTk, Tkinter
import Job, set_height

class Slice(Tkinter.Canvas):
    def __init__(self, parent, startrow, endrow, targetdir=None):
        Tkinter.Canvas.__init__(self, parent, width=800, height=50,
                                highlightthickness=0)
        self.statustext = self.create_text(5, 5, anchor="nw",
                                           font=("Helvetica", 16, "normal"))
        self.substatustext = self.create_text(5, 45, anchor="sw",
                                              font=("Helvetica", 10, "normal"))
        self.resourcetext = self.create_text(795, 25, anchor="e",
                                             font=("Helvetica", 24, "bold"))
        self.lineitem = self.create_line(5, 0, 795, 0)
        self.imageitem = None
        self.image = None
        self.pack(side="top")

        self.lock = threading.Semaphore()
        self.startrow = startrow
        self.endrow = endrow
        self.job = Job.Job(startrow, endrow, targetdir,
                           callback=self.statechange_callback,
                           sublock=sublock,
                           statlock=statlock,
                           getlock=getlock)
        self.state = None
        self.substate = None
        self.resource = None
        self.file = None
        self.running = 0

    def start(self):
        self.job.start()
        self.running = 1

    def statechange_callback(self, job):
        self.lock.acquire()
        self.state = job.state
        self.substate = job.substate
        self.resource = job.resource
        if job.state == "DOWNLOADED":
            self.file = job.outdir + "/monolith-%s-%s.png" % (self.startrow,
                                                              self.endrow)
        if job.substate == "DONE":
            self.running = 0
        self.lock.release()

    def update(self):
        if self.substate == "DONE" and self.file:
            if not self.imageitem:
                # POVray's output files are broken; fix the
                # height information
                set_height.set_height(self.file, 50)
                i = Image.open(self.file)
                self.image = ImageTk.PhotoImage(i)
                self.imageitem = self.create_image(0, 0, anchor="nw",
                                                   image=self.image)
        else:
            if self.state:
                self.itemconfigure(self.statustext, text=self.state)
            if self.substate:
                self.itemconfigure(self.substatustext, text=self.substate)
            else:
                self.itemconfigure(self.substatustext, text="")
            if self.resource:
                self.itemconfigure(self.resourcetext, text=self.resource)
        Tkinter.Canvas.update(self)

class Statusbar(Tkinter.Frame):
    def __init__(self, parent):
        self.parent = parent
        Tkinter.Frame.__init__(self, self.parent,
                               borderwidth=1, relief="raised")

        self.textitem = Tkinter.Label(self, anchor="w",
                                      font=("Helvetica", 10, "normal"))
        self.textitem.pack(side="left")

        self.quitbutton = Tkinter.Button(self, text="Quit",
                                         font=("Helvetica", 10, "normal"),
                                         command=self.quit,
                                         state="disabled")
        self.quitbutton.pack(side="right")

        self.pack(side="top", fill="both")


    def settext(self, text):
        self.textitem.configure(text=text)
        self.textitem.pack(side="left")



    def showquit(self):
        self.quitbutton.configure(state="normal")

class App:
    def __init__(self, root):
        self.root = root

        if not os.path.exists("out"):
            os.mkdir("out")

        self.slices = []
        for i in range(0, 600, 50):
            s = Slice(self.root, i+1, i+50, "out")
            self.slices.append(s)

        self.statusbar=Statusbar(self.root)

        for s in self.slices:
            s.start()

        self.update_callback()

    def update_callback(self):
        nrun = 0
        for s in self.slices:
            s.lock.acquire()
            s.update()
            if s.running:
                nrun += 1
            s.lock.release()

        self.statusbar.settext("%s jobs running" % nrun)

        if nrun == 0:
            self.statusbar.showquit()
        else:
            self.root.after(1000, self.update_callback)

# Max numbers of ngsub, ngstat and ngget processes running
# simultaneously
sublock = threading.Semaphore(4)
statlock = threading.Semaphore(5)
getlock = threading.Semaphore(3)

root = Tkinter.Tk()
root.title("POVray on the Grid")
app = App(root)

root.mainloop()
